import requests
from requests import Response

from .instance import Instance


class HttpRequests:

    def __init__(self):
        self._instance = Instance()

    @property
    def token(self):
        return self._get_token()

    def get_proposal(self, proposal_id: int):
        url = f"{self._instance.scicat_url}/Proposals/{proposal_id}?access_token={self.token}"
        response = requests.get(url, headers=self._instance.headers)
        if response.ok:
            return response.json()
        else:
            raise Exception(f"{response.text}")

    def push_dataset(self, dataset) -> Response:
        return requests.post(self._dataset_url(), headers=self._instance.headers, data=dataset.metadata)

    def update_dataset(self, dataset) -> Response:
        return requests.patch(self._dataset_url_by_pid(dataset.pid),
                              headers=self._instance.headers, data=dataset.metadata)

    def get_dataset(self, pid: str) -> dict:
        response = requests.get(self._dataset_url_by_pid(pid))
        if response.ok:
            return response.json()
        else:
            raise Exception(f"{response.text}")

    def delete_dataset(self, pid: str) -> None:
        """ Needs to be functional account: archiveManager or admin
        Response = {'error': {'statusCode': 401,
                              'name': 'Error',
                              'message': 'Authorization Required',
                              'code': 'AUTHORIZATION_REQUIRED'}}
        """
        response = requests.delete(f"{self._dataset_url_by_pid(pid)}")
        if response.ok:
            print(f"Dataset {pid} is deleted.")
        else:
            raise Exception(f"{response.text}")

    def get_all_datasets(self, proposal_id: int) -> list:
        response = requests.get(self._dataset_url())
        if response.ok:
            return [each for each in response.json() if each["proposalId"] == str(proposal_id)]
        else:
            raise Exception(f"{response.text}")

    def add_image_to_dataset(self, pid: str, image_request: str) -> Response:
        return requests.post(self._attachments_url_by_pid(pid),
                             headers=self._instance.headers, data=image_request)

    def _get_token(self) -> str:
        url = f"{self._instance.scicat_url}/Users/login"
        response = requests.post(url, headers=self._instance.headers,
                                 json={"username": self._instance.username,
                                       "password": self._instance.pw})
        if response.ok:
            return response.json()["id"]
        else:
            raise Exception(f"{response.text}")

    def _dataset_url(self) -> str:
        return f"{self._instance.scicat_url}/RawDatasets?access_token={self.token}"

    def _dataset_url_by_pid(self, pid: str) -> str:
        npid = self._reformat_pid(pid)
        return f"{self._instance.scicat_url}/RawDatasets/{npid}?access_token={self.token}"

    def _attachments_url_by_pid(self, pid: str) -> str:
        npid = self._reformat_pid(pid)
        return f"{self._instance.scicat_url}/Datasets/{npid}/Attachments?access_token={self.token}"

    @staticmethod
    def _reformat_pid(pid: str) -> str:
        """ SciCat adds a pid-prefix + "/"  but at DESY prefix = "" """
        return ("/" + pid).replace("/", "%2F")
