import json
from base64 import b64encode
from io import BytesIO

from matplotlib.figure import Figure


class ImageRequest:

    def __init__(self, dataset: dict, image: Figure, caption: str):
        self._dataset = dataset
        self._image = image
        self._caption = caption

    def create(self) -> str:
        b64_image = self._convert_image(self._image)
        message = {"ownerGroup": self._dataset['ownerGroup'],
                   "accessGroups": self._dataset['accessGroups'],
                   "caption": self._caption,
                   "thumbnail": f"data:image/png;base64,{b64_image}"}
        return json.dumps(message, indent=4)

    @staticmethod
    def _convert_image(image: Figure) -> str:
        s = BytesIO()
        image.savefig(s, format='png', bbox_inches="tight")
        return b64encode(s.getvalue()).decode("utf-8")
